class GuiActions:
    tab = "↔TAB"
    sair   = "Sair"
    editar = "Editar"
    rodar  = " Rodar"
    principal = "Principal"
    fixar  = "Fixar"
    tempo = "Tempo"
    hud = "HUD"
    github = "Github"
    ajuda = "Ajuda"
    baixar = "Baixar"
    ativar = "Ativar"
    navegar = "←↓→"
    marcar = "Marcar"
    desmarcar = "Desmarcar"
    colapsar = "Colapsar"
    pesquisar = "Buscar"


class GuiKeys:
    left = "a"
    right = "d"
    down = "s"
    up = "w"

    hud = "h"
    images = "I"
    down_task = "b"
    activate = "\n"
    key_help = "?"
    expand = ">"
    expand2 = "."
    collapse = "<"
    collapse2 = ","
    inc_grade = "+"
    inc_grade2 = "="
    dec_grade = "-"
    dec_grade2 = "_"
    set_root_dir = "D"
    set_lang = "L"
    github_open = "g"
    key_quit = "q"
    edit= "e"
    colors = "C"
    borders = "B"
    pesquisar = "/"
    graph = "G"
    diff = "m"
    rodar = "r"
    testar = "\n"
    principal = "\t"
    sair   = "q"
    editar = "e"
    travar = "f"
    tempo  = "t"
    border = "B"